#include <iostream.h>
#include <stdlib.h>
#include <math.h>

#define PI 3.1415927

int main(int argc, char** argv)
{
    /* Check arguments */
    if (argc != 2) {
	cerr << "Usage: " << argv[0] << " number_of_edgels\n";
	exit(-1);
    }
    int number = atoi(argv[1]);
    if (number <= 0) {
	cerr << "Number of edgels <= 0\n";
	exit(-1);
    }

    /* Allocate space */
    double *l1 = new double[number];
    double *l2 = new double[number];
    double *l3 = new double[number];
    double *cf = new double[number];
    double x,y,theta;
    double ex = 0;
    double ey = 0;
    for(int i=0; i<number; i++) {
	/* Read in data */
	cin >> x >> y >> theta >> cf[i];
	if (cin.bad()) {
	    cerr << "Error reading data\n";
	    exit(-1);
	}
	if (cin.eof()) {
	    cerr << "Number of edgels = " << i << "\n";
	    number = i;
	    continue;
	}

	/* Set line */
	theta*=PI/180.0;
	l1[i] = -sin(theta);
	l2[i] = cos(theta);
	l3[i] = x*sin(theta) - y*cos(theta);

        /* Check lines are aligned the correct way */
	if (l3[i] < 0.0) {
	    l1[i]=-l1[i];
	    l2[i]=-l2[i];
	    l3[i]=-l3[i];
	}

	/* Compute scaling information */
	ex += fabs(x);
	ey += fabs(y);
    }

    /* Check actually read in some data */
    if (number == 0) {
	cerr << "Number of edgels = 0\n";
	exit(-1);
    }
    else {
	ex = ex/number;
	ey = ey/number;
    }
    
    /* Compute confidence measure */
    double cf1 = 0.0;
    double cf2 = 0.0;
    double cf3 = 0.0;
    double cfs1 = 0.0;
    double cfs2 = 0.0;
    double cfs3 = 0.0;
    for(int i=0; i<number; i++) {
	cf1+=l1[i];
	cf2+=l2[i];
	cf3+=l3[i];
	cfs1+=l1[i]*l1[i];
	cfs2+=l2[i]*l2[i];
	cfs3+=l3[i]*l3[i];
	cout << cf[i] << " \t"
	     << (fabs(cfs1-cf1*cf1/(i+1))*ex*ex +
		 fabs(cfs2-cf2*cf2/(i+1))*ey*ey +
		 fabs(cfs3-cf3*cf3/(i+1)))/(ex*ey)/(i+1)
	     << " \n";
    }

    /* Clean up and exit */
    delete [] l1;
    delete [] l2;
    delete [] l3;
    return 0;
}
