#include <iostream.h>
#include <stdlib.h>
#include <math.h>

#define PI 3.1415927
#define SEP 5.0*5.0

int main(int argc, char** argv)
{
    /* Check arguments */
    if (argc != 2) {
	cerr << "Usage: " << argv[0] << " number_of_edgels\n";
	exit(-1);
    }
    int number = atoi(argv[1]);
    if (number <= 0) {
	cerr << "Number of edgels <= 0\n";
	exit(-1);
    }

    /* Allocate space */
    double *x = new double[number];
    double *y = new double[number];
    double *cf = new double[number];
    double theta;
    double ex = 0;
    double ey = 0;
    for(int i=0; i<number; i++) {
	/* Read in data */
	cin >> x[i] >> y[i] >> theta >> cf[i];
	if (cin.bad()) {
	    cerr << "Error reading data\n";
	    exit(-1);
	}
	if (cin.eof()) {
	    cerr << "Number of edgels = " << i << "\n";
	    number = i;
	    continue;
	}

	/* Compute scaling information */
	ex += fabs(x[i]);
	ey += fabs(y[i]);
    }

    /* Check actually read in some data */
    if (number == 0) {
	cerr << "Number of edgels = 0\n";
	exit(-1);
    }
    else {
	ex = ex/number;
	ey = ey/number;
    }
    
    /* Compute confidence measure */
    double cf1 = 0.0;
    double cf2 = 0.0;
    double cf3 = 0.0;
    double cfs1 = 0.0;
    double cfs2 = 0.0;
    double cfs3 = 0.0;
    int sum = 0;
    for(int i=0; i<number; i++) {
	for(int j=0; j<i; j++) {
	    /* Check separation of edgels */
	    if (((x[i]-x[j])*(x[i]-x[j])+(x[i]-x[j])*(x[i]-x[j])) > SEP) {
		/* Compute line segement between ith and jth edgel*/
		double theta=atan2(y[j]-y[i],x[j]-x[i]);
		double l1 = -sin(theta);
		double l2 = cos(theta);
		double l3 = x[i]*sin(theta) - y[i]*cos(theta);

		/* Check line aligned the correct way */
		if (l3 < 0.0) {
		    l1=-l1;
		    l2=-l2;
		    l3=-l3;
		}

		/* Update confidence measure */
		cf1+=l1;
		cf2+=l2;
		cf3+=l3;
		cfs1+=l1*l1;
       		cfs2+=l2*l2;
		cfs3+=l3*l3;
		sum+=1;
	    }
	}

	/* Output, scaling to acount for no. of pairs used in computation */
	cout << cf[i] << " \t"
	     << ((sum == 0) ? 0.0 : (1.0/((double) sum) *
		 (fabs(cfs1-cf1*cf1/(sum))*ex*ex +
		  fabs(cfs2-cf2*cf2/(sum))*ey*ey +
		  fabs(cfs3-cf3*cf3/(sum)))/(ex*ey)))
	     << " \n";
    }

    /* Clean up and exit */
    delete [] x;
    delete [] y;
    return 0;
}


