#include <iostream.h>
#include <stdlib.h>
#include <math.h>

#define PI 3.1415927
#define SEP 15

int main(int argc, char** argv)
{
    /* Check arguments */
    if (argc != 2) {
	cerr << "Usage: " << argv[0] << " number_of_edgels\n";
	exit(-1);
    }
    int number = atoi(argv[1]);
    if (number <= 0) {
	cerr << "Number of edgels <= 0\n";
	exit(-1);
    }

    /* Allocate space */
    double *l1 = new double[number];
    double *l2 = new double[number];
    double *l3 = new double[number];
    double *th = new double[number];
    double *cf = new double[number];
    double x,y,theta,th2;
    double ex = 0;
    double ey = 0;
    for(int i=0; i<number; i++) {
	/* Read in data */
	cin >> x >> y >> theta >> cf[i];
	th2 = theta;
	if (cin.bad()) {
	    cerr << "Error reading data\n";
	    exit(-1);
	}
	if (cin.eof()) {
	    cerr << "Number of edgels = " << i << "\n";
	    number = i;
	    continue;
	}
	
	/* Set line */
	theta*=PI/180.0;
	l1[i] = -sin(theta);
	l2[i] = cos(theta);
	l3[i] = x*sin(theta) - y*cos(theta);

	/* Check range of theta */
        while(th2<0.0)
            th2+=360.0;
        while(th2>360.0)
            th2-=360.0;
        if (th2>=180.0)
            th2-=180.0;
	th[i] = th2;

	/* Compute scaling information */
	ex += fabs(x);
	ey += fabs(y);
    }

    /* Check actually read in some data */
    if (number == 0) {
	cerr << "Number of edgels = 0\n";
	exit(-1);
    }
    else {
	ex = ex/number;
	ey = ey/number;
    }
    
    /* Compute confidence measure */
    double cf1 = 0.0;
    double cf2 = 0.0;
    double cfs1 = 0.0;
    double cfs2 = 0.0;
    int sum = 0;
    for(int i=0; i<number; i++) {
	for(int j=0; j<i; j++) {
	    /* Check separation of edgels */
	    if (fabs(th[i]-th[j]) > SEP &&
		fabs(th[i]-th[j]+180) > SEP &&
		fabs(th[i]-th[j]-180) > SEP) {
		/* Compute the intersection of the ith and jth edgel */
		double v1 = l2[i]*l3[j] - l3[i]*l2[j];
		double v2 = l3[i]*l1[j] - l1[i]*l3[j];
		double v3 = l1[i]*l2[j] - l2[i]*l1[j];
		if (v3 == 0.0) {
		    /* Should never reach here */
		    cerr << "Unexpected divide by zero\n";
		    exit(-1);
		}

		/* Update confidence measure */
		v1/=v3;
		v2/=v3;
		cf1+=v1;
		cf2+=v2;
		cfs1+=v1*v1;
		cfs2+=v2*v2;
		sum+=1;
	    }
	}

	cout << cf[i] << " \t"
	     << ((sum == 0) ? 0.0 : (1.0/((double) sum) *
		 (fabs(cfs1-cf1*cf1/sum)/ex/ex +
		  fabs(cfs2-cf2*cf2/sum)/ey/ey)))
	     << " \n";
    }

    /* Clean up and exit */
    delete [] l1;
    delete [] l2;
    delete [] l3;
    delete [] th;
    return 0;
}
