#include <iostream.h>
#include <stdlib.h>
#include <math.h>

#define PI 3.1415927
#define SEP 20

int main(int argc, char** argv)
{
    /* Check arguments */
    if (argc != 2) {
	cerr << "Usage: " << argv[0] << " number_of_edgels\n";
	exit(-1);
    }
    int number = atoi(argv[1]);
    if (number <= 0) {
	cerr << "Number of edgels <= 0\n";
	exit(-1);
    }

    /* Allocate space */
    double *l1 = new double[number];
    double *l2 = new double[number];
    double *l3 = new double[number];
    double *cf = new double[number];
    double x,y,theta;
    for(int i=0; i<number; i++) {
	/* Read in data */
	cin >> x >> y >> theta >> cf[i];
	if (cin.bad()) {
	    cerr << "Error reading data\n";
	    exit(-1);
	}
	if (cin.eof()) {
	    cerr << "Number of edgels = " << i << "\n";
	    number = i;
	    continue;
	}
	
	/* Set line */
	theta*=PI/180.0;
	l1[i] = -sin(theta);
	l2[i] = cos(theta);
	l3[i] = x*sin(theta) - y*cos(theta);
    }
    
    /* Compute confidence measure */
    double cf1 = 0.0;
    double cfs1 = 0.0;
    int sum = 0;
    for(int i=0; i<number; i++) {
	for(int j=0; j<i; j++) {
	    /* Check separation of edgels */
	    if (fabs(l3[i]-l3[j]) > SEP) {
		/* Compute dirn of vanishing point of ith and jth edgels */
		double v1 = l2[i]*l3[j] - l3[i]*l2[j];
		double v2 = l3[i]*l1[j] - l1[i]*l3[j];
		double angle = 180.0*atan2(v1,v2)/PI;
		if (angle < 0) {
		    angle+=180.0;
		}

		/* Update confidence measure */
		cf1+=angle;
		cfs1+=angle*angle;
		sum+=1;
	    }
	}

	cout << cf[i] << " \t"
	     << ((sum == 0) ? 0.0 : (1.0/((double) sum) *
		 fabs(cfs1-cf1*cf1/sum)/180.0/180.0))
	     << " \n";
    }

    /* Clean up and exit */
    delete [] l1;
    delete [] l2;
    delete [] l3;
    return 0;
}
