#include <iostream.h>
#include <stdlib.h>
#include <math.h>

#define PI 3.1415927
#define iterations 10000
#define max_sum 100
#define SEP 35.0*35.0
#define SEP2 0.707

int guass_jordan(double *a, double *b, double *c);

int main(int argc, char** argv)
{
    /* Check arguments */
    if (argc != 2) {
	cerr << "Usage: " << argv[0] << " number_of_edgels\n";
	exit(-1);
    }
    int number = atoi(argv[1]);
    if (number <= 0) {
	cerr << "Number of edgels <= 0\n";
	exit(-1);
    }

    /* Allocate space */
    double *x = new double[number];
    double *y = new double[number];
    double *theta = new double[number];
    double *conf = new double[number];
    double ex = 0;
    double ey = 0;
    for(int i=0; i<number; i++) {
	/* Read in data */
	cin >> x[i] >> y[i] >> theta[i] >> conf[i];
	
	/* Check status of input stream */
	if (cin.bad()) {
	    cerr << "Error reading data\n";
	    exit(-1);
	}
	if (cin.eof()) {
	    cerr << "Number of edgels = " << i << "\n";
	    number = i;
	    continue;
	}

	/* Compute scaling information */
	ex += fabs(x[i]);
	ey += fabs(y[i]);
    }

    /* Check actually read in some data */
    if (number == 0) {
	cerr << "Number of edgels = 0\n";
	exit(-1);
    }
    else {
	ex = ex/number;
	ey = ey/number;
    }
    
    /* Compute confidence measure */
    double cf[5];
    double cfs[5];
    double ocf[5];
    double ocfs[5];
    for(int k=0; k<5; k++) {
	cf[k]=cfs[k]=0.0;
    }
    double a[25];
    double b[5];
    double c[5];
    int sum=0;
    int osum=0;
    int ind[5];
    srand(982312);                     /* Initialize random number generator */
    for(int i=0; i<number; i++) {
	/* Reset everything */
        for(int k=0; k<5; k++) {
	    cf[k]=cfs[k]=0.0;
	}
	sum = 0;

	/* Can't afford an n^5 algorithm - use Monte Carlo */
	for(int j=0; j<iterations && sum<max_sum; j++) {

	    /* Randomly choose 5 edgels */
	    for(int k=0; k<5; k++) {
		ind[k]=rand() % (i+1);
	    }

	    /* Check separation of edgels */
	    int sep=1;
	    for(int k=0; k<5; k++) {
		for(int l=k+1; l<5; l++) {
		    if ((x[ind[l]]-x[ind[k]])*(x[ind[l]]-x[ind[k]]) +
			(y[ind[l]]-y[ind[k]])*(y[ind[l]]-y[ind[k]]) < SEP) {
			sep = 0;
		    }
		}
	    }
	    if (sep) {
		sep = 0;
		for(int k=0; k<5; k++) {
		    for(int l=k+1; l<5; l++) {
			for(int m=l+1; m<5; m++) {
			    double a=sqrt((x[ind[k]]-x[ind[l]])*(x[ind[k]]-x[ind[l]])+
					  (y[ind[k]]-y[ind[l]])*(y[ind[k]]-y[ind[l]]));
			    double b=sqrt((x[ind[k]]-x[ind[m]])*(x[ind[k]]-x[ind[m]])+
					  (y[ind[k]]-y[ind[m]])*(y[ind[k]]-y[ind[m]]));
			    double c=sqrt((x[ind[m]]-x[ind[l]])*(x[ind[m]]-x[ind[l]])+
					  (y[ind[m]]-y[ind[l]])*(y[ind[m]]-y[ind[l]]));
			    if ((fabs((b*b+c*c-a*a)/2/b/c) < SEP2) &&
				(fabs((a*a+c*c-b*b)/2/a/c) < SEP2) &&
				(fabs((b*b+a*a-c*c)/2/b/a) < SEP2)) {
				sep = 1;
			    }
			}
		    }
		}
	    }

	    /* If far enough apart */
	    if (sep) {
		/* Set up matrices */
		for(int k=0; k<5; k++) {
		    b[k]=1.0;
		    a[5*k+0]=x[ind[k]]*x[ind[k]];
		    a[5*k+1]=x[ind[k]];
		    a[5*k+2]=y[ind[k]]*y[ind[k]];
		    a[5*k+3]=y[ind[k]];
		    a[5*k+4]=x[ind[k]]*y[ind[k]];
		}

		/* If Gauss-Jordan worked */
		if (guass_jordan(a,b,c)) {
		    /* Update the confidence measure */
		    for(int k=0; k<5; k++) {
			cf[k]+=c[k];
			cfs[k]+=c[k]*c[k];
		    }
		    sum+=1;
		}
	    }
	}

	/* Output, scaling to acount for no. of triples used in computation */
	if (sum < max_sum) {
	    if (osum) {
		cout << conf[i] << " \t"
		     << (1.0/((double) osum) *
			 (fabs(ocfs[0]-ocf[0]*ocf[0]/osum)*ex*ex*ex*ex +
			  fabs(ocfs[1]-ocf[1]*ocf[1]/osum)*ex*ex +
			  fabs(ocfs[2]-ocf[2]*ocf[2]/osum)*ey*ey*ey*ey +
			  fabs(ocfs[3]-ocf[3]*ocf[3]/osum)*ey*ey +
			  fabs(ocfs[4]-ocf[4]*ocf[4]/osum)*ex*ey*ex*ey))
		     << " \n";
	    }
	    else {
		cout << conf[i] << " \t0.0\n";
	    }
	}
	else {
	    for(int k=0;k<5; k++) {
		ocfs[k]=cfs[k];
		ocf[k]=cf[k];
	    }
	    osum = sum;
	    cout << conf[i] << " \t"
		 << (1.0/((double) sum) *
		     (fabs(cfs[0]-cf[0]*cf[0]/sum)*ex*ex*ex*ex +
		      fabs(cfs[1]-cf[1]*cf[1]/sum)*ex*ex +
		      fabs(cfs[2]-cf[2]*cf[2]/sum)*ey*ey*ey*ey +
		      fabs(cfs[3]-cf[3]*cf[3]/sum)*ey*ey +
		      fabs(cfs[4]-cf[4]*cf[4]/sum)*ex*ey*ex*ey))
		 << " \n";
	}
    }
    
    /* Clean up and exit */
    delete [] x;
    delete [] y;
    delete [] conf;
    delete [] theta;
    return 0;
}

int guass_jordan(double *a, double *b, double *c) 
{
    int r[5];
    for(int k=0; k<5; k++) {
	r[k]=k;
    }

    /* Perform Gauss-Jordan */
    for(int k=0; k<5; k++) {
	double big=0.0;
	int big_row=k;
	for(int l=k; l<5; l++) {
	    if (fabs(a[5*r[l]+k]) > big) {
		big = fabs(a[5*r[l]+k]);
		big_row = l;
	    }
	}
	if (big == 0.0) {
	    cerr << "Matrix singular\n";
	    return 0;
	}

	/* Switch the rows */
	int tmp=r[k];
	r[k]=r[big_row];
	r[big_row]=tmp;;

	for(int l=k+1; l<5; l++) {
	    a[5*r[k]+l]/=a[5*r[k]+k];
	}
	b[r[k]]/=a[5*r[k]+k];
	a[5*r[k]+k]=1.0;
	for(int l=k+1; l<5; l++) {
	    for(int m=k+1; m<5; m++) {
		a[5*r[l]+m]-=a[5*r[l]+k]*a[5*r[k]+m];
	    }
	    b[r[l]]-=a[5*r[l]+k]*b[r[k]];
	    a[5*r[l]+k]=0.0;
	}
    }
    for(int k=3; k>=0; k--) {
	for(int l=k+1; l<5; l++) {
	    b[r[k]]-=a[5*r[k]+l]*b[r[l]];
	}
    }
    for(int k=0; k<5; k++) {
	c[k]=b[r[k]];
    }

    return 1;
}
