#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>
#include <math.h>

int main(int argc, char** argv)
{
    /* Check arguments */
    if (argc != 4) {
	cerr << "Usage: " << argv[0] << " number_of_edgels file1 file2\n";
	exit(-1);
    }
    int edgels = atoi(argv[1]);
    if (edgels <= 0) {
	cerr << "Number of edgels <= 0\n";
	exit(-1);
    }

    /* Allocate space */
    int files=2;
    ifstream input;
    double *data = new double[edgels*files];
    double x,y;

    /* Read in the data */
    int max_valid = edgels;
    for(int i=0; i<files; i++) {
	/* Open the ith file */
	input.open(argv[2+i]);

	/* Read in the confidence measure */
	for(int j=0;j<edgels; j++) {
	    input >> x >> y;
	    if (input.bad()) {
		cerr << "Error reading file: " << argv[2+i] << "\n";
		exit(-1);
	    }
	    if (input.eof()) {
		cerr << "Number of edgels in file: " << argv[2+i]
		     << " = " << j << "\n";
		if (max_valid > j)
		    max_valid = j;
		break;
	    }

	    data[i*edgels+j] = y;
	}

	/* Close ith file */
	input.close();
    }
    
    /* Output the results */
    for(int j=0; j<max_valid; j++) {
        double sum = data[j]+data[edgels+j];
        double diff = data[j]-data[edgels+j];
	if (sum == 0.0) {
	    cout << j+1 << " \t0.0\n";
	}
	else {
	    cout << j+1 << " \t" << diff / sum << "\n";
	}
    }

    /* Clean up and exit */    
    delete [] data;
    return 0;
}
